import numpy as np

class Circle:
    '''A class for circles
      Constructor is initialized with two tuples, one for the center of the circle
      and the other for a point on the circle.
      
      Methods include radius, area, and circum.  None of these methods accept any arguments.
      
      The user is not required to pre-compute the radius of the circle.  Exception testing is 
      done in area and circum to check for a circle radius.  If it doesn't exist, a radius is 
      computed.
    '''
    
    def __init__(self, center, point):
        self.xc = center[0]
        self.yc = center[1]
        self.x = point[0]
        self.y = point[1]
    
    def radius(self):
        x = self.x - self.xc
        y = self.y - self.yc
        self.R = np.sqrt(x * x + y * y)
        return self.R
    
    def area(self):
        try:
            self.A = np.pi * self.R* self.R
        except AttributeError:
            r = self.radius()
            self.A = np.pi * r * r
        return self.A
    
    def circum(self):
        try:
            self.C =  2.0 * np.pi * self.R
        except AttributeError:
            r = self.radius()
            self.C = 2.0 * np.pi * r
        return self.C
    
    def __eq__(self, other):
        return self.R==other.R

class Rcircle(Circle):
    def __init__(self, r):
        self.R = r
    
    def radius(self):
        return self.R


if __name__ == "__main__":

    # Create some circles
    circle1 = Circle((np.sqrt(2), np.sqrt(2)), (1.0, 0.0))
    circle2 = Rcircle(2.0)
    
    # Print the radius of each circle
    print(circle1.radius())
    print(circle2.radius())
    
    # Check to see if the two circles are the same size
    print(circle1==circle2)

    # Print the area of each circle
    # (to make sure Rcircle inherits from Circle correctly)
    print(circle1.area())
    print(circle2.area())

    # Print the circumference of each circle
    # (to make sure Rcircle inherits from Circle correctly)
    print(circle1.circum())
    print(circle2.circum())

    circle3 = Circle((1.0, 0.0), (3.0, 0.0))
    
    # Print the radius of each circle
    print(circle3.radius())
    print(circle2.radius())
    
    # Check to see if the two circles are the same size
    print(circle3==circle2)
